@@ -2,6 +2,7 @@ from __future__ import annotations
 
 import difflib
 import hashlib
+import shutil
 
 from collections import namedtuple
 from collections.abc import Iterable, Sequence
@@ -60,6 +61,15 @@ def _normalize_git_ssh(url: str) -> str:
     return reconstruct_url(urlparts)
 
 
+def normalize_git_url(url: str) -> str:
+    """Normalize HTTP(s) and SSH Git URLs"""
+    if "@" in url and ("://" not in url or url.startswith("ssh://")):
+        url = _normalize_git_ssh(url)
+    elif url.startswith("http://") or url.startswith("https://"):
+        url = url_normalize(url)
+    return url
+
+
 def _colorize_diff(line: str) -> str:
     if line.startswith("--- ") or line.startswith("+++ ") or line.startswith("@@ "):
         return click.style(line, fg="yellow")
@@ -167,11 +177,12 @@ class GitRepo:
         author_name: Optional[str] = None,
         author_email: Optional[str] = None,
         config=None,
+        bare=False,
     ):
         if not force_init and targetdir.exists():
             self._repo = Repo(targetdir)
         else:
-            self._repo = Repo.init(targetdir)
+            self._repo = Repo.init(targetdir, bare=bare)
 
         if remote:
             self.remote = remote
@@ -198,10 +209,7 @@ class GitRepo:
 
     @remote.setter
     def remote(self, remote: str):
-        if "@" in remote and ("://" not in remote or remote.startswith("ssh://")):
-            remote = _normalize_git_ssh(remote)
-        elif remote.startswith("http://") or remote.startswith("https://"):
-            remote = url_normalize(remote)
+        remote = normalize_git_url(remote)
         try:
             self._repo.remote().set_url(remote)
         except ValueError:
@@ -227,6 +235,15 @@ class GitRepo:
         sha = self._repo.head.commit.hexsha
         return self._repo.git.rev_parse(sha, short=6)
 
+    @property
+    def _author_env(self) -> dict[str, str]:
+        return {
+            Actor.env_author_name: self._author.name or "",
+            Actor.env_author_email: self._author.email or "",
+            Actor.env_committer_name: self._author.name or "",
+            Actor.env_committer_email: self._author.email or "",
+        }
+
     @property
     def _null_tree(self) -> Tree:
         """Generate empty Tree for the repo.
@@ -286,8 +303,171 @@ class GitRepo:
 
         return CommitInfo(commit=commit, branch=branch, tag=tag)
 
+    def fetch(
+        self, remote: str = "origin", tags: bool = True, prune: bool = True
+    ) -> Iterable[FetchInfo]:
+        return self._repo.remote(remote).fetch(tags=tags, prune=prune)
+
+    def has_local_branches(self) -> bool:
+        local_heads = set(h.name for h in self.repo.heads)
+        remote_prefix = self._remote_prefix()
+        remote_heads = set(h.name.replace(remote_prefix, "", 1) for h in self.fetch())
+        return len(local_heads - remote_heads) > 0
+
+    def has_local_changes(self) -> bool:
+        return self._repo.is_dirty() or len(self._repo.untracked_files) > 0
+
+    def _create_worktree(self, worktree: Path, version: str):
+        """Create worktree.
+
+        This method expects `worktree` to not exist."""
+
+        # We need to use `git.execute()` for the worktree commands as GitPython only has
+        # basic support for worktrees.
+        self._repo.git.execute(["git", "worktree", "prune"])
+        cmd = ["git", "worktree", "add", "-f", str(worktree), version]
+        try:
+            self._repo.git.execute(cmd)
+        except GitCommandError as e:
+            # Assume that GitCommandError is only caused by invalid versions
+            raise RefError(f"Failed to checkout revision '{version}'") from e
+
+    def _migrate_to_worktree(self, wtr: GitRepo, worktree: Path, version: str):
+        """Migrate non-worktree checkout to worktree."""
+        if wtr.has_local_branches() or wtr.has_local_changes():
+            raise click.ClickException(
+                f"Migrating dependency {worktree} to worktree-based checkout "
+                + "would delete uncommitted changes, untracked files, or "
+                + "unpushed branches, aborting..."
+            )
+        click.secho(f" > Removing non-worktree based checkout {worktree}", fg="yellow")
+        shutil.rmtree(worktree)
+        self._create_worktree(worktree, version)
+
+    def _update_worktree_remote(self, wtr: GitRepo, worktree: Path, version: str):
+        """Update existing worktree checkout to new remote.
+
+        Updating the remote for a worktree needs special handling, since we generally
+        don't want to update the remote URL for the previously used remote URL's bare
+        clone, but instead recreate the worktree from the new remote's bare clone."""
+        if wtr.has_local_changes():
+            raise click.ClickException(
+                f"Switching remote for worktree '{worktree}' would delete "
+                + "uncommitted changes or untracked files, aborting..."
+            )
+        # Remove stale worktree if there are no uncommitted changes
+        wtr_common_dir = Path(wtr.repo.common_dir).resolve()
+        click.secho(
+            f" > Removing stale, but clean worktree {worktree}, "
+            + f"bare repo is available at {wtr_common_dir}",
+            fg="green",
+        )
+        wtr.repo.git.execute(["git", "worktree", "remove", str(worktree)])
+        self._create_worktree(worktree, version)
+
+    def _checkout_existing_worktree(self, worktree: Path, version: str):
+        """Perform checkout if requested worktree directory already exists.
+
+        The heavy work is generally done by `_migrate_to_worktree()`,
+        `_update_worktree_remote()` or `checkout()`.
+        """
+        wtr = GitRepo(None, worktree)
+        if not wtr.repo.has_separate_working_tree():
+            # If the worktree's common dir is stored in the repository working tree
+            # root, we're migrating from a non-worktree checkout to a worktree checkout.
+            self._migrate_to_worktree(wtr, worktree, version)
+        elif wtr.remote != self.remote:
+            # If the existing directory is already a worktree, but we're using a
+            # different remote for the requested worktree, we need to recreate the
+            # worktree from the new remote's bare clone.
+            self._update_worktree_remote(wtr, worktree, version)
+        else:
+            # Otherwise, we just need to update the worktree's version. We simply use
+            # `checkout()` in the worktree to do so.
+            wtr.checkout(version)
+
+    def checkout_worktree(self, worktree: Path, version: Optional[str]):
+        """Create worktree if it doesn't exist and check out `version` in it.
+
+        If `version` is not provided, the remote's default branch is checked out.
+
+        If a repo which isn't a worktree of `self` is found in the requested worktree
+        location, the method will try to replace the old checkout with the requested
+        worktree unless there's any local changes (untracked files, uncommitted changes,
+        or branches which don't exist upstream).
+        """
+        # Try to fetch remote heads, so we can actually check them out
+        try:
+            _ = self.fetch()
+        except ValueError:
+            pass
+
+        if version is None:
+            version = self._default_version()
+
+        # If the worktree directory exists, use `_checkout_existing_worktree()`
+        if worktree.is_dir():
+            self._checkout_existing_worktree(worktree, version)
+            return
+
+        # If the worktree directory doesn't exist yet, create the worktree
+        self._create_worktree(worktree, version)
+
+    def initialize_worktree(
+        self, worktree: Path, initial_branch: Optional[str] = None
+    ) -> None:
+        if not initial_branch:
+            initial_branch = self._default_version()
+
+        # We need an initial commit to be able to create a worktree. Create initial
+        # commit from empty tree.
+        initsha = self._repo.git.execute(  # type: ignore[call-overload]
+            command=[
+                "git",
+                "commit-tree",
+                "-m",
+                "Initial commit",
+                self._null_tree.hexsha,
+            ],
+            env=self._author_env,
+        )
+
+        # Create worktree using the provided branch name
+        self._repo.git.execute(
+            ["git", "worktree", "add", str(worktree), initsha, "-b", initial_branch]
+        )
+
+    @property
+    def worktrees(self) -> list[GitRepo]:
+        """List all worktrees for the repo"""
+        # First prune worktrees, to ensure repo worktree state is clean
+        self._repo.git.execute(["git", "worktree", "prune"])
+        worktrees: list[GitRepo] = []
+        wt_list = self._repo.git.execute(
+            ["git", "worktree", "list", "--porcelain"],
+            as_process=False,
+            with_extended_output=False,
+            stdout_as_string=True,
+        ).splitlines()
+        for line in wt_list:
+            if " " not in line:
+                continue
+            k, v = line.split(" ")
+            if k == "worktree":
+                worktrees.append(GitRepo(None, Path(v)))
+
+        return worktrees
+
     def checkout(self, version: Optional[str] = None):
-        remote_heads = self._repo.remote().fetch(prune=True, tags=True)
+        remote_heads = self.fetch()
+        if not remote_heads:
+            # GitPython's fetch-info parsing chokes on lines like
+            # "   (refs/remotes/origin/HEAD has become dangling)"
+            # see also https://github.com/gitpython-developers/GitPython/issues/962.
+            # We handle this case by simply performing a second fetch which should
+            # return fetch-infos with flags = 4 (HEAD_UPTODATE).
+            remote_heads = self.fetch()
+
         if version is None:
             # Handle case where we want the default branch of the remote
             version = self._default_version()
@@ -366,13 +546,28 @@ class GitRepo:
         """Add provided list of files to index."""
         self._repo.index.add(files)
 
-    def commit(self, commit_message: str):
+    def commit(self, commit_message: str, amend=False):
         author = self._author
 
         if self._trace:
             click.echo(f' > Using "{author.name} <{author.email}>" as commit author')
 
-        self._repo.index.commit(commit_message, author=author, committer=author)
+        if amend:
+            # We need to call out to `git commit` for amending
+            self._repo.git.execute(  # type: ignore[call-overload]
+                [
+                    "git",
+                    "commit",
+                    "--amend",
+                    "--no-edit",
+                    "--reset-author",
+                    "-m",
+                    commit_message,
+                ],
+                env=self._author_env,
+            )
+        else:
+            self._repo.index.commit(commit_message, author=author, committer=author)
 
     def push(
         self, remote: Optional[str] = None, version: Optional[str] = None
